/* SwingML
 * Copyright (C) 2002 Robert Morris.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors:
 *     Robert Morris <robertj@morris.net>
 *
 */

package org.swingml.component;

import java.awt.Component;
import java.awt.Container;

import org.swingml.model.GridBagCellModel;


/**
 * This class exists only within a hierarchy of GridBag* components.  The levels 
 * of this hierarchy are as follows:
 * 
 * GridBagPanelComponent/
 * 	GridBagRowComponent/
 * 		GridBagCellComponent
 * 
 * Only the GridBagPanelComponent is actually drawn and displayed on the screen at run-time.
 * The GridBagRowComponent only contains references to other GridBagCellComponents and some
 * formatting instructions that apply to all objects it references.  The GridBagCellComponent
 * contains formatting instructions, just like the GridBagRowComponent, that override the settings
 * specified by its parent.  The GridBagCellComponent also contains other Component
 * instances that are displayed within the GridBagPanelComponent, laid out according
 * to the instructions specified by this hierarchy.
 * 
 * @see org.swingml.component.GridBagPanelComponent
 * @see org.swingml.component.GridBagRowComponent
 */
public class GridBagCellComponent extends Container {
    
    private GridBagPanelComponent m_parentPanel = null;
    private GridBagCellModel m_model = null;
    private int m_column = -1;

    /**
     * This method indicates the x coordinate of the contained component
     * within a GridBagLayout.  This property is set by the containing
     * GridBagRowComponent's add() method.
     * 
     * @param	column	The x coordinate of the contained component 
     * within a GridBagLayout.
     */
    public void setColumn(int aColumn) {
        this.m_column = aColumn;
    }

    /**
     * This method sets the parent panel to the value specified by the
     * parentPanel argument.  The parent panel represents an actual GridBagPanelComponent
     * where the components contained by this instance are displayed.  
     * 
     * @param	parentPanel	A valid reference to an instance of the GridBagPanelComponent class.
     */
    public void setParentPanel(GridBagPanelComponent aParentPanel) {
        this.m_parentPanel = aParentPanel;
    }

    /**
     * This add() method override takes the specified component, and adds it to the 
     * GridBagPanelComponent instance that was set in the @see #setParentPanel(GridBagPanelComponent) 
     * method.  The GridBagLayout constraints contained within the parent GridBagRowComponent
     * and this instance are applied to the specified component when it is added to the 
     * parentPanel.
     * 
     * @param	component	A valid reference to an instance of the Component class, or a derivative.
     * 
     * @return	The component specified in the component parameter.
     * 
     * @see org.swingml.component.GridBagRowComponent
     * @see org.swingml.component.GridBagPanelComponent
     */
    public Component add(Component aComponent) {
        if (this.m_parentPanel != null) {
            this.m_parentPanel.add(aComponent, this.m_model.getConstraints());
        }

        return aComponent;
    }

    /**
     * Constructor for GridBagCellComponent.
     */
    public GridBagCellComponent(GridBagCellModel aModel) {
        this.m_model = aModel;
    }
}