/* SwingML
 * Copyright (C) 2002 Ezequiel Cuellar.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors:
 *     Ezequiel Cuellar <ecuellar@crosslogic.com>
 *
 */

package org.swingml.component;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JComponent;
import javax.swing.JDialog;

import org.swingml.Constants;
import org.swingml.RenderingThread;
import org.swingml.SubmittingThread;
import org.swingml.event.EventHandler;
import org.swingml.model.JDialogModel;


public class JDialogComponent extends JDialog implements WindowListener {

    private EventHandler eventHandler = EventHandler.getInstance();
    private JDialogModel model = null;
    private Container owner = null;

    public JDialogComponent(JDialogModel aModel) {
        this.model = aModel;
        String theName = model.getName();
        String theLayout = model.getLayout();
        String theText = model.getText();
        int theRows = model.getRows();
        int theCols = model.getCols();
        int theWidth = model.getWidth();
        int theHeight = model.getHeight();
        super.setName(theName);
        super.setTitle(theText);
        super.setSize(theWidth, theHeight);
        super.setModal(model.isModal());
        if (theLayout.equalsIgnoreCase(Constants.FLOWLAYOUT)) {
            super.getContentPane().setLayout(new FlowLayout());
        }
        if (theLayout.equalsIgnoreCase(Constants.GRIDLAYOUT)) {
            super.getContentPane().setLayout(new GridLayout(theRows, theCols));
        }
        if (theLayout.equalsIgnoreCase(Constants.BORDERLAYOUT)) {
            super.getContentPane().setLayout(new BorderLayout());
        }
        if (theLayout.equalsIgnoreCase(Constants.NONE)) {
            super.getContentPane().setLayout(null);
        }

        super.addWindowListener(this);
    }

    public void render(String aUrl, String aParent) {
        RenderingThread theThread = new RenderingThread(aUrl, this.getContentPane());
        theThread.start();
    }

    public void submit(String anAddress) {
        SubmittingThread theThread = new SubmittingThread(anAddress, this);
        theThread.start();
    }

    public void setOwner(Container aContainer) {
        this.owner = aContainer;
    }

    public Container getDialogOwner() {
        return this.owner;
    }

    private JComponent getFirstJComponent(Container aContainer) {
        JComponent theComponent = null;
        if (aContainer != null && aContainer.getComponents() != null && aContainer.getComponents().length > 0) {
            Component[] theComponents = aContainer.getComponents();
            for (int i = 0; i < theComponents.length; i++) {
                if (theComponents[i] instanceof JComponent) {
                    theComponent = (JComponent) theComponents[i];
                    break;
                } else if (theComponents[i] instanceof Container) {
                    theComponent = getFirstJComponent((Container) theComponents[i]);
                    if (theComponent != null) {
                        break;
                    }
                }
            }
        }
        return theComponent;
    }

    // WindowListener Realization.
    private void handleEvent(String eventType) {
        /*
         * This code makes sure that a JComponent instance exists in the current DIALOG
         * element.  If not, no event is fired.  If so, the event is fired with the 
         * first instance of a JComponent class located within the current dialog's 
         * ContentPane.
         */
        JComponent theComponent = this.getFirstJComponent(this.getContentPane());
        if (theComponent != null) {
            this.eventHandler.handleEvent(this.model, eventType, theComponent);
        }
    }

    /**
     * @see java.awt.event.WindowListener#windowActivated(WindowEvent)
     */
    public void windowActivated(WindowEvent aEvt) {
        this.handleEvent(Constants.WINDOW_ACTIVATED);
    }

    /**
     * @see java.awt.event.WindowListener#windowClosed(WindowEvent)
     */
    public void windowClosed(WindowEvent aEvt) {
        this.handleEvent(Constants.WINDOW_CLOSED);
    }

    /**
     * @see java.awt.event.WindowListener#windowClosing(WindowEvent)
     */
    public void windowClosing(WindowEvent aEvt) {
        this.handleEvent(Constants.WINDOW_CLOSING);
    }

    /**
     * @see java.awt.event.WindowListener#windowDeactivated(WindowEvent)
     */
    public void windowDeactivated(WindowEvent aEvt) {
        this.handleEvent(Constants.WINDOW_DEACTIVATED);
    }

    /**
     * @see java.awt.event.WindowListener#windowDeiconified(WindowEvent)
     */
    public void windowDeiconified(WindowEvent aEvt) {
        this.handleEvent(Constants.WINDOW_DEICONIFIED);
    }

    /**
     * @see java.awt.event.WindowListener#windowIconified(WindowEvent)
     */
    public void windowIconified(WindowEvent aEvt) {
        this.handleEvent(Constants.WINDOW_ICONIFIED);
    }

    /**
     * @see java.awt.event.WindowListener#windowOpened(WindowEvent)
     */
    public void windowOpened(WindowEvent aEvt) {
        this.handleEvent(Constants.WINDOW_OPENED);
    }
}