/* SwingML
 * Copyright (C) 2002 Ezequiel Cuellar.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors:
 *     Ezequiel Cuellar <ecuellar@crosslogic.com>
 *
 */

package org.swingml.component;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;

import javax.swing.JList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.swingml.Constants;
import org.swingml.XMLTranslatable;
import org.swingml.event.EventHandler;
import org.swingml.model.ItemModel;
import org.swingml.model.JListModel;


public class JListComponent extends JList implements ListSelectionListener, XMLTranslatable, DropTargetListener, DragSourceListener, DragGestureListener, MouseListener {
    private JListModel model = null;
    private DropTarget dropTarget = null;
    private DragSource dragSource = null;
    private Object selected = null;
    private EventHandler eventHandler = EventHandler.getInstance();

    public JListComponent(JListModel theModel) {
        this.model = theModel;
        super.setListData(model.getChildren().toArray());
        super.setName(model.getName());
        super.setToolTipText(model.getTooltip());
        super.setSelectedIndices(model.getSelectedIndexes());
        super.setSelectionMode(model.getMode());
        super.addListSelectionListener(this);
        super.addMouseListener(this);
        if (theModel.isDndEnabled()) {
            this.dropTarget = new DropTarget(this, this);
            this.dragSource = new DragSource();
            this.dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_MOVE, this);
        }
    }

    public void setListModel(JListModel aListModel) {
        this.model = aListModel;
        super.setListData(aListModel.getChildren().toArray());
    }

    //-- ListSelectionListener realization.
    public void valueChanged(ListSelectionEvent aEvt) {
        this.eventHandler.handleEvent(this.model, Constants.LIST_VALUE_CHANGED, this);
    }

    //-- XMLTranslatable realization.
    public String getXMLValue() {
        Object[] theValues = null;
        StringBuffer theResult = new StringBuffer();
        if (this.getListModel().getPostStyle().equalsIgnoreCase(Constants.POST_ALL)) {
            theValues = this.getListModel().getChildren().toArray();
        } else {
            // Constants.POST_SELECTED
            theValues = this.getSelectedValues();
        }

        if (theValues != null && theValues.length > 0) {
            for (int i = 0; i < theValues.length; i++) {
                // Determine whether or not the original SwingML document indicated a VALUE attribute
                // for the ITEM tag.  If so, return that value rather than the value of the TEXT attribute.
                if (theValues[i] instanceof ItemModel && ((ItemModel) theValues[i]).getValue() != null) {
                    theResult.append(((ItemModel) theValues[i]).getValue());
                } else {
                    theResult.append(theValues[i].toString());
                }

                if (i < theValues.length - 1) {
                    theResult.append(",");
                }
            }
        }
        return theResult.toString();
    }

    //-- DragSourceListener realization.	
    public void dragDropEnd(DragSourceDropEvent aEvt) {
        if (aEvt.getDropSuccess()) {
            removeElement();
        }
    }

    public void dragEnter(DragSourceDragEvent aEvt) {
    }

    public void dragOver(DragSourceDragEvent aEvt) {
    }

    public void dropActionChanged(DragSourceDragEvent aEvt) {
    }

    public void dragExit(DragSourceEvent aEvt) {
    }

    //-- DragGestureListener realization.
    public void dragGestureRecognized(DragGestureEvent aEvt) {
        this.selected = getSelectedValue();
        if (selected != null) {
            StringSelection theText = new StringSelection(selected.toString());
            dragSource.startDrag(aEvt, DragSource.DefaultMoveDrop, theText, this);
        }
    }

    //-- DropTargetListener realization.
    public void drop(DropTargetDropEvent aEvt) {
        try {
            Transferable theTransferable = aEvt.getTransferable();
            if (theTransferable.isDataFlavorSupported(DataFlavor.stringFlavor)) {
                aEvt.acceptDrop(DnDConstants.ACTION_MOVE);
                String theText = (String) theTransferable.getTransferData(DataFlavor.stringFlavor);
                addElement(theText);
                aEvt.getDropTargetContext().dropComplete(true);
            } else {
                aEvt.rejectDrop();
            }
        } catch (IOException e) {
            System.out.println(e);
            aEvt.rejectDrop();
        } catch (UnsupportedFlavorException e) {
            System.out.println(e);
            aEvt.rejectDrop();
        }
    }

    public void dragEnter(DropTargetDragEvent aEvt) {
        aEvt.acceptDrag(DnDConstants.ACTION_MOVE);
    }

    public void dropActionChanged(DropTargetDragEvent aEvt) {
    }

    public void dragOver(DropTargetDragEvent aEvt) {
    }

    public void dragExit(DropTargetEvent aEvt) {
    }

    //-- Utility methods.
    public JListModel getListModel() {
        return this.model;
    }

    public void removeElement() {
        JListModel theListModel = this.getListModel();
        theListModel.getChildren().remove(selected);
        super.setListData(theListModel.getChildren().toArray());
    }

    public void addElement(Object anElement) {
        ItemModel theNewItem = new ItemModel();
        theNewItem.setText(anElement.toString());
        JListModel theListModel = this.getListModel();
        theListModel.addChild(theNewItem);
        super.setListData(theListModel.getChildren().toArray());
    }

    // MouseListener Realization.
    /**
     * @see java.awt.event.MouseListener#mouseClicked(MouseEvent)
     */
    public void mouseClicked(MouseEvent aEvt) {
        final int DOUBLE_CLICK_COUNT = 2;
        final int SINGLE_CLICK_COUNT = 1;
        String theEventType = null;
        switch (aEvt.getClickCount()) {
        case DOUBLE_CLICK_COUNT:
            theEventType = Constants.MOUSE_DOUBLE_CLICKED;
            break;

        case SINGLE_CLICK_COUNT:
            theEventType = Constants.MOUSE_SINGLE_CLICKED;
            break;

        default:
            theEventType = null;
            break;
        }
        if (theEventType != null && theEventType.length() > 0) {
            this.eventHandler.handleEvent(this.model, theEventType, this);
        }
    }

    /**
     * @see java.awt.event.MouseListener#mouseEntered(MouseEvent)
     */
    public void mouseEntered(MouseEvent aEvt) {
    }

    /**
     * @see java.awt.event.MouseListener#mouseExited(MouseEvent)
     */
    public void mouseExited(MouseEvent aEvt) {
    }

    /**
     * @see java.awt.event.MouseListener#mousePressed(MouseEvent)
     */
    public void mousePressed(MouseEvent aEvt) {
    }

    /**
     * @see java.awt.event.MouseListener#mouseReleased(MouseEvent)
     */
    public void mouseReleased(MouseEvent aEvt) {
    }
}