/* SwingML
 * Copyright (C) 2002 Ezequiel Cuellar.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors:
 *     Ezequiel Cuellar <ecuellar@crosslogic.com>
 *     Farid Ibrahim <faridibrahim@lycos.com>
 * 
 */

package org.swingml.component;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import org.swingml.Constants;
import org.swingml.XMLTranslatable;
import org.swingml.event.EventHandler;
import org.swingml.model.JTextFieldModel;


public class JTextFieldComponent extends JTextField implements XMLTranslatable, DocumentListener, FocusListener, MouseListener {

    private JTextFieldModel model = null;
    private EventHandler eventHandler = EventHandler.getInstance();

    public JTextFieldComponent(JTextFieldModel aModel) {
        this.model = aModel;
        super.setName(aModel.getName());
        super.setText(aModel.getText());
        super.setColumns(Integer.parseInt(aModel.getCols()));
        super.setEditable(model.isEditable());
        super.getDocument().addDocumentListener(this);
        super.addFocusListener(this);
        super.setToolTipText(aModel.getTooltip());
        super.setEnabled(aModel.isEnabled());
        super.addMouseListener(this);
        if (aModel.getForeground() != null) {
            super.setForeground(aModel.getForeground());
        }
        if (aModel.getBackground() != null) {
            super.setBackground(aModel.getBackground());
        }
    }

    public JTextFieldModel getModel() {
        return this.model;
    }

    public void removeUpdate(DocumentEvent aEvt) {
        this.eventHandler.handleEvent(this.getModel(), Constants.REMOVE_UPDATE, this);
    }

    public void insertUpdate(DocumentEvent aEvt) {
        this.eventHandler.handleEvent(this.getModel(), Constants.INSERT_UPDATE, this);
    }

    public void changedUpdate(DocumentEvent aEvt) {
        this.eventHandler.handleEvent(this.getModel(), Constants.CHANGED_UPDATE, this);
    }

    public void focusGained(FocusEvent aEvt) {
        this.eventHandler.handleEvent(this.getModel(), Constants.FOCUS_GAINED, this);
    }

    public void focusLost(FocusEvent aEvt) {
        this.eventHandler.handleEvent(this.getModel(), Constants.FOCUS_LOST, this);
    }

    public String getXMLValue() {
        return super.getText();
    }

    // MouseListener Realization

    public void mouseClicked(MouseEvent aEvt) {
        possiblePopup(aEvt);
    }

    public void mouseEntered(MouseEvent aEvt) {
    }

    public void mouseExited(MouseEvent aEvt) {
    }

    public void mousePressed(MouseEvent aEvt) {
        possiblePopup(aEvt);
    }

    public void mouseReleased(MouseEvent aEvt) {
        possiblePopup(aEvt);
    }

    public void possiblePopup(MouseEvent aEvt) {
        if (aEvt.isPopupTrigger()) {
            ContextMenu thePopup = new ContextMenu(this);
            thePopup.show(aEvt.getComponent(), aEvt.getX(), aEvt.getY());
        }
    }

    private class ContextMenu extends JPopupMenu implements ActionListener {
        private final String MENU_CUT = "Cut";
        private final String MENU_COPY = "Copy";
        private final String MENU_PASTE = "Paste";
        private final String MENU_DELETE = "Delete";
        private final String MENU_SEPARATOR = "-";
        private final String MENU_SELECT_ALL = "Select All";

        private JTextFieldComponent m_textField = null;

        public ContextMenu(JTextFieldComponent textField) {
            this.m_textField = textField;

            String items[] = { MENU_CUT, MENU_COPY, MENU_PASTE, MENU_DELETE, MENU_SEPARATOR, MENU_SELECT_ALL };

            for (int i = 0; i < items.length; i++) {
                if (items[i].equalsIgnoreCase(MENU_SEPARATOR)) {
                    javax.swing.JSeparator separator = new JSeparator();
                    this.add(separator);
                } 
                else {
                    JMenuItem menuItem = new JMenuItem(items[i]);
                    menuItem.addActionListener(this);

                    boolean isEnabled = ((items[i].equalsIgnoreCase(MENU_CUT) || items[i].equalsIgnoreCase(MENU_COPY) || items[i].equalsIgnoreCase(MENU_DELETE)) && (this.m_textField.getSelectedText() == null || this.m_textField.getSelectedText().length() == 0) ? false : true);

                    // This entire block is used just to determine whether or not the 
                    // data in the clipboard is available for pasting into a text field.
                    if (items[i].equalsIgnoreCase(MENU_PASTE)) {
                        try {
                            Clipboard clipBoard = this.m_textField.getToolkit().getSystemClipboard();

                            if (clipBoard != null) {
                                Transferable clipObject = clipBoard.getContents(null);
                                Object realData = clipObject.getTransferData(DataFlavor.stringFlavor);
                                isEnabled = (realData != null);
                            }
                        } catch (UnsupportedFlavorException e) {
                            isEnabled = false;
                        } catch (Exception e) {
                            // Swallow the exception.
                        }
                    } 
                    // Finally set the enabled status of the menu item.
                    menuItem.setEnabled(isEnabled);
                    this.add(menuItem);
                } 
            } 
        } 

        public void actionPerformed(ActionEvent event) {
            if (event.getActionCommand().equalsIgnoreCase(MENU_CUT)) {
                this.m_textField.cut();
            } else if (event.getActionCommand().equalsIgnoreCase(MENU_COPY)) {
                this.m_textField.copy();
            } else if (event.getActionCommand().equalsIgnoreCase(MENU_PASTE)) {
                this.m_textField.paste();

            } else if (event.getActionCommand().equalsIgnoreCase(MENU_DELETE)) {
                if (this.m_textField.getSelectedText() != null && this.m_textField.getSelectedText().length() > 0) {
                    StringBuffer text = new StringBuffer(this.m_textField.getText());

                    text.replace(this.m_textField.getSelectionStart(), this.m_textField.getSelectionEnd(), "");
                    this.m_textField.setText(text.toString());
                }
            } else if (event.getActionCommand().equalsIgnoreCase(MENU_SELECT_ALL)) {
                this.m_textField.requestFocus();
                this.m_textField.selectAll();
            }
        } 
    } 
}