/* SwingML
 * Copyright (C) 2002 Ezequiel Cuellar.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors:
 *     Ezequiel Cuellar <ecuellar@crosslogic.com>
 *
 */

package org.swingml.component;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceDragEvent;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DragSourceEvent;
import java.awt.dnd.DragSourceListener;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.util.Enumeration;

import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.TransferHandler;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeWillExpandListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.ExpandVetoException;
import javax.swing.tree.TreePath;

import org.swingml.Constants;
import org.swingml.SwingMLModel;
import org.swingml.XMLTranslatable;
import org.swingml.event.EventHandler;
import org.swingml.model.JTreeModel;


public class JTreeComponent extends JTree implements XMLTranslatable, DropTargetListener, DragSourceListener, TreeSelectionListener, DragGestureListener, MouseListener, TreeExpansionListener, TreeWillExpandListener {

    private JTreeModel model = null;
    private StringBuffer xmlValue = null;
    private DragSource dragSource = null;
    private DefaultMutableTreeNode targetNode = null;
    private DefaultMutableTreeNode sourceNode = null;
    private TreePath eventPath = null;
    private EventHandler eventHandler = EventHandler.getInstance();

    public JTreeComponent(TreeNodeComponent aNode, JTreeModel aModel) {
        super(aNode);
        this.model = aModel;
        super.setName(model.getName());
        super.addTreeSelectionListener(this);
        super.addMouseListener(this);
        if (model.isDndEnabled()) {
            DropTarget theDropTarget = new DropTarget(this, DnDConstants.ACTION_MOVE, this, true);
            this.dragSource = DragSource.getDefaultDragSource();
            this.dragSource.createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_MOVE, this);
        }
        super.addTreeExpansionListener(this);
        super.addTreeWillExpandListener(this);
    }

    //-- XMLTranslatable realization.
    public String getXMLValue() {
        this.xmlValue = new StringBuffer();
        this.processTreeNode((TreeNodeComponent) super.getModel().getRoot());
        return this.xmlValue.toString();
    }

    public void processTreeNode(TreeNodeComponent aNode) {

        this.xmlValue.append("<TREE NAME=\"" + aNode.getName() + " \" TEXT=\"" + aNode.getUserObject() + "\">\n");
        TreeNodeComponent theChild = null;
        int theCount = aNode.getChildCount();
        for (int i = 0; i < theCount; i++) {
            theChild = (TreeNodeComponent) aNode.getChildAt(i);
            if (!theChild.isLeaf()) {
                this.processTreeNode(theChild);
            } else {
                this.xmlValue.append("<NODE TEXT=\"" + theChild.getUserObject() + "\"/>\n");
            }
        }
        this.xmlValue.append("</TREE>\n");
    }

    //-- TreeSelectionListener realization.
    public void valueChanged(TreeSelectionEvent aEvt) {
        if (aEvt.getNewLeadSelectionPath() != null) {
            this.sourceNode = (DefaultMutableTreeNode) aEvt.getNewLeadSelectionPath().getLastPathComponent();
        }
        this.eventHandler.handleEvent(this.model, Constants.TREE_VALUE_CHANGED, this);
    }

    //-- DragGestureListener realization.
    public void dragGestureRecognized(DragGestureEvent aEvt) {
        if (this.sourceNode != super.getModel().getRoot()) {
            this.dragSource.startDrag(aEvt, DragSource.DefaultMoveNoDrop, (Transferable) this.sourceNode, this);
        }
    }

    //-- DragSourceListener realization.
    public void dragEnter(DragSourceDragEvent aEvt) {
        setCursor(aEvt);
    }

    public void dragOver(DragSourceDragEvent aEvt) {
        setCursor(aEvt);
    }

    public void dropActionChanged(DragSourceDragEvent aEvt) {
        setCursor(aEvt);
    }

    public void dragExit(DragSourceEvent aEvt) {
        setCursor(aEvt);
    }

    public void dragDropEnd(DragSourceDropEvent aEvt) {
        super.updateUI();
    }

    //-- DropTargetListener realization.
    public void drop(DropTargetDropEvent aEvt) {
        try {
            if (this.isDropValid(aEvt)) {
                this.sourceNode = (DefaultMutableTreeNode) aEvt.getTransferable().getTransferData(TreeNodeComponent.dataFlavor);
                if (this.sourceNode.isLeaf()) {
                    /*
                     * If the node being dragged is leaf then it will be added after the 
                     * last leaf node of the current parent.
                     * */
                    //-- targetNode is set in isValid() method.
                    Enumeration theChildren = this.targetNode.children();
                    DefaultMutableTreeNode theIterationNode = null;
                    int theIndex = 0;
                    while (theChildren.hasMoreElements()) {
                        theIterationNode = (DefaultMutableTreeNode) theChildren.nextElement();
                        if (theIterationNode.isLeaf()) {
                            theIndex = targetNode.getIndex(theIterationNode);
                        }
                    }
                    if (this.sourceNode.getParent() != this.targetNode && theIndex > 0) {
                        theIndex++;
                    }
                    targetNode.insert(this.sourceNode, theIndex);
                } else {
                    /*
                     * If the node being dragged is not leaf the it will be added at the end of
                     * the current parent.
                     * */
                    this.targetNode.add(this.sourceNode);
                }
                aEvt.acceptDrop(TransferHandler.MOVE);
                aEvt.dropComplete(true);
                super.updateUI();
            } else {
                aEvt.rejectDrop();
            }
        } catch (IllegalArgumentException e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    JOptionPane.showMessageDialog(JTreeComponent.this, "Can't drag a folder onto itself or one of its children.");
                }
            });
        } catch (UnsupportedFlavorException e) {
            System.out.println(e);
        } catch (IOException e) {
            System.out.println(e);
        }
    }

    public void dragEnter(DropTargetDragEvent aEvt) {
        if (this.isDropValid(aEvt)) {
            aEvt.acceptDrag(DnDConstants.ACTION_MOVE);
        } else {
            aEvt.rejectDrag();
        }
    }

    public void dropActionChanged(DropTargetDragEvent aEvt) {
        if (this.isDropValid(aEvt)) {
            aEvt.acceptDrag(DnDConstants.ACTION_MOVE);
        } else {
            aEvt.rejectDrag();
        }
    }

    public void dragOver(DropTargetDragEvent aEvt) {
        if (isDropValid(aEvt)) {
            aEvt.acceptDrag(DnDConstants.ACTION_MOVE);
        } else {
            aEvt.rejectDrag();
        }
    }

    public void dragExit(DropTargetEvent aEvt) {
    }

    //-- Utility methods.
    private boolean isDropValid(DropTargetEvent aEvt) {
        Point thePoint = null;

        if (aEvt instanceof DropTargetDropEvent) {
            thePoint = ((DropTargetDropEvent) aEvt).getLocation().getLocation();
        } else if (aEvt instanceof DropTargetDragEvent) {
            thePoint = ((DropTargetDragEvent) aEvt).getLocation().getLocation();
        }

        TreePath thePath = getPathForLocation(thePoint.x, thePoint.y);
        if (thePath != null) {
            //-- Sets the targetNode;
            this.targetNode = (DefaultMutableTreeNode) thePath.getLastPathComponent();
            if (this.targetNode.isLeaf()) {
                return false;
            } else {
                return true;
            }
        }
        return false;
    }

    public void setCursor(DragSourceEvent aEvt) {
        if (aEvt instanceof DragSourceDragEvent) {
            Cursor theResult = null;
            int theDropAction = ((DragSourceDragEvent) aEvt).getDropAction();
            int theTargetActions = ((DragSourceDragEvent) aEvt).getUserAction();
            if ((theDropAction & theTargetActions) != 0) { //-- Able to drop
                if (theDropAction == DnDConstants.ACTION_MOVE) {
                    theResult = DragSource.DefaultMoveDrop;
                }
            } else { //-- Unable to drop
                if (theDropAction == DnDConstants.ACTION_MOVE) {
                    theResult = DragSource.DefaultMoveNoDrop;
                }
            }
            aEvt.getDragSourceContext().setCursor(theResult);
        } else {
            aEvt.getDragSourceContext().setCursor(DragSource.DefaultMoveNoDrop);
        }
    }

    // MouseListener Realization
    /**
     * @see java.awt.event.MouseListener#mouseClicked(MouseEvent)
     */
    public void mouseClicked(MouseEvent aEvt) {
        final int DOUBLE_CLICK_COUNT = 2;
        final int SINGLE_CLICK_COUNT = 1;
        String theEventType = null;
        switch (aEvt.getClickCount()) {
        case DOUBLE_CLICK_COUNT:
            theEventType = Constants.MOUSE_DOUBLE_CLICKED;
            break;
        case SINGLE_CLICK_COUNT:
            theEventType = Constants.MOUSE_SINGLE_CLICKED;
            break;
        default:
            theEventType = null;
            break;
        }
        if (theEventType != null && theEventType.length() > 0) {
            this.eventHandler.handleEvent(this.model, theEventType, this);
        }
    }

    /**
     * @see java.awt.event.MouseListener#mouseEntered(MouseEvent)
     */
    public void mouseEntered(MouseEvent aEvt) {
    }

    /**
     * @see java.awt.event.MouseListener#mouseExited(MouseEvent)
     */
    public void mouseExited(MouseEvent aEvt) {
    }

    /**
     * @see java.awt.event.MouseListener#mousePressed(MouseEvent)
     */
    public void mousePressed(MouseEvent aEvt) {
    }

    /**
     * @see java.awt.event.MouseListener#mouseReleased(MouseEvent)
     */
    public void mouseReleased(MouseEvent aEvt) {
    }

    public TreePath getEventPath() {
        return this.eventPath;
    }

    // TreeExpansionListener Realization
    public void treeCollapsed(TreeExpansionEvent aEvt) {
        this.eventPath = aEvt.getPath();
        this.eventHandler.handleEvent(this.model, Constants.TREE_COLLAPSED, this);
    }

    public void treeExpanded(TreeExpansionEvent aEvt) {
        this.eventPath = aEvt.getPath();
        this.eventHandler.handleEvent(this.model, Constants.TREE_EXPANDED, this);
    }

    // TreeWillExpandListener Realization
    public void treeWillCollapse(TreeExpansionEvent aEvt) throws ExpandVetoException {
        this.eventPath = aEvt.getPath();
        this.eventHandler.handleEvent(this.model, Constants.TREE_WILL_COLLAPSE, this);
    }

    public void treeWillExpand(TreeExpansionEvent e) throws ExpandVetoException {
        this.eventPath = e.getPath();
        eventHandler.handleEvent((SwingMLModel) this.model, Constants.TREE_WILL_EXPAND, this);
    }
}