/* SwingML
 * Copyright (C) 2002 Robert Morris.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors:
 *     Robert Morris <robertj@morris.net>
 *
 */

package org.swingml.event;

import java.awt.Component;
import java.awt.Container;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JComponent;

import org.swingml.SwingMLRenderer;
import org.swingml.model.ActionParamModel;


/**
 * This class provides utility functions for implementers of the 
 * InvokableEvent class.  Generally, this class is the super class
 * of most InvokableEvent implementations
 * 
 * @author Robert J. Morris
 * 
 * @see org.swingml.event.InvokableEvent
 * @see org.swingml.model.ActionParamModel
 */
public class EventUtil {
    /**
     * This method expects the objects in the params argument
     * to be instances of the ActionParamModel object.  
     * 
     * @param params A reference to an array of Object instances.
     * 
     * @return A valid reference to a Map object.  null if an error occurred.
     * 
     * @see org.swingml.model.ActionParamModel
     */
    protected Map buildParamMap(Object[] aParams) {
        Map theReturnMap = null;
        ActionParamModel theParam = null;
        if (aParams != null && aParams.length > 0) {
            theReturnMap = new HashMap(aParams.length);
            for (int i = 0; i < aParams.length; i++) {
                if (aParams[i] instanceof ActionParamModel) {
                    theParam = (ActionParamModel) aParams[i];
                    theReturnMap.put(theParam.getName(), theParam.getValue());
                }
            }
        }
        return theReturnMap;
    }

    private Container getRootComponent(Component aComponent) {
        Container theRetComp = null;
        if (aComponent instanceof JComponent) {
            theRetComp = (Container) ((JComponent) aComponent).getTopLevelAncestor();
        } else if (aComponent.getParent() == null && aComponent instanceof Container) {
            theRetComp = (Container) aComponent;
        } else if (aComponent != null) {
            theRetComp = getRootComponent(aComponent.getParent());
        }
        return theRetComp;
    }

    /**
     * This method returns a reference to the Renderer instance 
     * within which the Component referenced by the component argument
     * resides.
     * 
     * @param component A refernce to the component specified
     * by the COMPONENT attribute of the &lt;EXTERNAL-ACTION&gt; tag.
     * 
     * @return A reference to the SwingMLRender within which
     * the InvokableEvent instance resides.
     */
    protected SwingMLRenderer getRenderer(Component aComponent) {
        SwingMLRenderer theRet = null;
        try {
            Container theRootComponent = this.getRootComponent(aComponent);
            if (!(theRootComponent instanceof SwingMLRenderer)) {
                theRet = this.findRenderer(theRootComponent.getComponents());
            } else {
                theRet = (SwingMLRenderer) theRootComponent;
            }
        } catch (Exception e) {
            System.out.println("Syntax Error: There is a problem resolving the SwingMLRenderer for the referenced component.");
        }
        return theRet;
    }

    private SwingMLRenderer findRenderer(Component[] aComponents) {
        SwingMLRenderer theRetComp = null;
        SwingMLRenderer theFoundComp = null;
        if (aComponents != null && aComponents.length > 0) {
            for (int i = 0; i < aComponents.length; i++) {
                if (aComponents[i] instanceof SwingMLRenderer) {
                    theRetComp = (SwingMLRenderer) aComponents[i];
                    break;
                }
                if (aComponents[i] instanceof Container) {
                    theFoundComp = this.findRenderer(((Container) aComponents[i]).getComponents());
                    if (theFoundComp != null) {
                        theRetComp = theFoundComp;
                        break;
                    }
                }
            }
        }
        return theRetComp;
    }

    private Component findComponent(Component[] aComponents, String aComponentName) {
        Component theRetComp = null;
        Component theFoundComp = null;
        if (aComponents != null && aComponents.length > 0) {
            for (int i = 0; i < aComponents.length; i++) {
                if (aComponents[i].getName() != null && aComponents[i].getName().equalsIgnoreCase(aComponentName)) {
                    theRetComp = aComponents[i];
                    break;
                }
                if (aComponents[i] instanceof Container) {
                    theFoundComp = this.findComponent(((Container) aComponents[i]).getComponents(), aComponentName);
                    if (theFoundComp != null) {
                        theRetComp = theFoundComp;
                        break;
                    }
                }
            }
        }
        return theRetComp;
    }

    /**
     * This function returns a reference to the component specified by the 
     * componentName argument.  The component argument does not need to contain
     * the requested object, it only needs to be in the same document as the 
     * component argument.
     * 
     * @param component A reference to the Component instance
     * specified in the COMPONENT attribute of the &lt;EXTERNAL-ACTION&gt; tag.
     * 
     * @param componentName A String representing the name of the 
     * component to return.
     * 
     * @return A reference to the named component, or null if the 
     * name does not exist within the document hierarchy.
     */
    protected Component getComponent(Component aComponent, String aComponentName) {
        Component theRet = null;
        Container theRootComponent = this.getRootComponent(aComponent);
        theRet = this.findComponent(theRootComponent.getComponents(), aComponentName);
        return theRet;
    }
}