/* SwingML
 * Copyright (C) 2002 Robert Morris.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors:
 *     Robert Morris <robertj@morris.net>
 *
 */

package org.swingml.event;

import java.util.HashMap;
import java.util.Map;

import org.swingml.SwingMLRenderer;


/**
 *
 * This class provides a cache of ExternalEvent implementations.  
 * When an ExternalEvent implementation implements the RetainedInMemory
 * Interface and returns true from the retainInMemory() method, this class
 * will create one instance and re-use for the lifetime of the current
 * document.  Once the document is released, the cache of objects is 
 * flushed.
 * 
 * @author <a href="mailto:robertj@morris.net">Robert J. Morris</a>
 */
public class ExternalEventManager {
    protected static Map globalExternalEvents = null;
    protected static Map documents = null;

    /**
     * This method forces all cached ExternalEvent instances to
     * be released.
     */
    public static void flush() {
        if (ExternalEventManager.globalExternalEvents != null) {
            ExternalEventManager.globalExternalEvents.clear();
        }

        if (ExternalEventManager.documents != null) {
            ExternalEventManager.documents.clear();
        }
    }

    /**
     * This method flushes the InvokableEvent cache associated 
     * with the referenced renderer parameter.  This will release
     * all InvokableEvent instances that were cached using
     * RetainedInMemory.retainInMemory() == RetainedInMemory.MEM_DOCUMENT.
     * Globally stored events are left intact.  Events associated with 
     * other renderer instances are also left intact.
     */
    public static void flush(SwingMLRenderer aRenderer) {
        if (ExternalEventManager.documents != null) {
            if (ExternalEventManager.documents.containsKey(aRenderer)) {
                ExternalEventManager.documents.remove(aRenderer);
            }
        }
    }

    /**
     * This method returns an instance of the InvokableEvent implementation
     * as specified by the className argument.  If the referenced class does not
     * implement the InvokableEvent interface, a value of null is returned.
     * If the referenced class also implements the RetainedInMemory interface 
     * and returns true for the retainInMemory() method, it will either retrieve
     * an existing instance from static memory, or create a new instance and
     * store it in static memory.
     * 
     * @param className The name of the InvokableEvent implementation to instnatiate.
     * 
     * @return A valid reference to the class described in the className argument.
     */
    public static InvokableEvent loadInvokableEvent(SwingMLRenderer aRenderer, String aClassName) {
        InvokableEvent theInvokableEvent = null;
        Map theDocumentEvents = null;
        try {
            if (ExternalEventManager.globalExternalEvents == null) {
                ExternalEventManager.globalExternalEvents = new HashMap(1);
            }
            if (ExternalEventManager.documents == null) {
                ExternalEventManager.documents = new HashMap(1);
            }
            // Determine whether or not the class has been cached 
            // at the document-level of the referenced renderer
            // instance.
            if (ExternalEventManager.documents.containsKey(aRenderer)) {
                theDocumentEvents = (Map) ExternalEventManager.documents.get(aRenderer);
                if (theDocumentEvents.containsKey(aClassName)) {
                    theInvokableEvent = (InvokableEvent) theDocumentEvents.get(aClassName);
                }
            }
            if (theInvokableEvent == null) {
                if (ExternalEventManager.globalExternalEvents.containsKey(aClassName)) {
                    theInvokableEvent = (InvokableEvent) ExternalEventManager.globalExternalEvents.get(aClassName);
                } else {
                    theInvokableEvent = (InvokableEvent) Class.forName(aClassName).newInstance();
                    if (theInvokableEvent instanceof RetainedInMemory) {
                        int theMemoryStyle = ((RetainedInMemory) theInvokableEvent).retainInMemory();
                        if (theMemoryStyle == RetainedInMemory.MEM_GLOBAL) {
                            ExternalEventManager.globalExternalEvents.put(aClassName, theInvokableEvent);
                        } else if (theMemoryStyle == RetainedInMemory.MEM_DOCUMENT) {
                            // The current instance of the InvokableEvent implementation
                            // has specified that it should be cached on a per-document
                            // basis.  To do this, we must first obtain a document-level
                            // event cache.  Then we add the new object to it.
                            if (theDocumentEvents == null) {
                                theDocumentEvents = new HashMap(1);
                                ExternalEventManager.documents.put(aRenderer, theDocumentEvents);
                            }
                            theDocumentEvents.put(aClassName, theInvokableEvent);
                        } else if (theMemoryStyle == RetainedInMemory.MEM_VOLATILE) {
                            // DO NOTHING.  MEM_VOLATILE is the same as not implementing RetainedInMemory.
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } 
        return theInvokableEvent;
    }
} 
