/* SwingML
 * Copyright (C) 2002 Robert Morris.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors:
 *     Robert Morris <robertj@morris.net>
 *
 */

package org.swingml.event;

import java.awt.Component;

/**
 * Represents the basic functionality necessary to create a SwingML 
 * invokable event.  An InvokableEvent is any Java class that provides
 * event-driven functionality outside of that provided by the renderer.
 *
 * @author <a href="mailto:robertj@morris.net">Robert J. Morris</a>
 * 
 * @see org.swingml.event.EventHandler
 * @see org.swingml.event.ExternalEventManager
 * @see org.swingml.event.RetainedInMemory
 */
public interface InvokableEvent {
    /**
     * This method provides an initialization routine for an
     * InvokableEvent implementation.  This method is called upon 
     * every invocation of the event, regardless of whether or 
     * not the implementation is retained in memory.
     * 
     * @param component A reference the component specified in the 
     * &lt;EXTERNAL-ACTION&gt; tag's COMPONENT attribute.  
     * 
     * @param params An array of ActionParamModel objects.  
     */
    public void initialize(Component aComponent, Object[] aParams);

    /**
     * This method provides the actual action performed by the 
     * InvokableEvent implementation.  Called immediately after the 
     * initialize() method.
     */
    public void invoke();

    /**
     * This method is called immediately after the completion of the 
     * invoke() method.  This method should be used to release any 
     * allocated resource (database, files, etc) allocated during
     * the initialize() or invoke() methods.
     */
    public void destroy();
}