/* SwingML
 * Copyright (C) 2002 Robert Morris.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors:
 *     Robert Morris <robertj@morris.net>
 *
 */
package org.swingml.model;

import java.awt.GridBagConstraints;
import java.awt.Insets;

import org.swingml.SwingMLModel;


/**
 * This class is the parent of all GridBag model components.  It contains properties
 * that correspond to the settings contained with the GridBagConstraints class.  These
 * values are set by the GridBagBaseMapper class during the parsing of a SwingML document.
 * 
 * @author <a href="mailto:robertj@morris.net">Robert J. Morris</a>
 * 
 * @see org.swingml.model.GridBagCellModel
 * @see org.swingml.model.GridBagRowModel
 * @see org.swingml.model.GridBagPanelModel
 * @see java.awt.GridBagConstraints
 */
public class GridBagBaseModel extends SwingMLModel {
    protected String m_name = null;
    protected int m_padding = 0;
    protected int m_ipadx = 0;
    protected int m_ipady = 0;
    protected int m_fill = GridBagConstraints.NONE;
    protected int m_anchor = GridBagConstraints.CENTER;
    protected double m_weightx = 0;
    protected double m_weighty = 0;
    protected int m_gridheight = 1;
    protected int m_gridwidth = 1;

    /**
     * This method returns a valid reference to an instance of the @see java.awt.GridBagConstraints
     * class with attribute values set according to the property values contained
     * within the current instance.
     * 
     * @return	A valid reference to an instance of the GridBagConstraints class.
     */
    public GridBagConstraints getConstraints() {
        GridBagConstraints theCon = new GridBagConstraints();
        theCon.insets = new Insets(m_padding, m_padding, m_padding, m_padding);
        theCon.ipadx = this.getIpadx();
        theCon.ipady = this.getIpady();
        theCon.fill = this.getFill();
        theCon.anchor = this.getAnchor();
        theCon.weightx = this.getWeightx();
        theCon.weighty = this.getWeighty();
        theCon.gridheight = this.getGridHeight();
        theCon.gridwidth = this.getGridWidth();

        return theCon;
    }

    /**
     * This method takes the values contained within the specified baseModel parameter
     * to set the values of the current instance.  
     * 
     * @param	baseModel	A reference to another GridBagBaseModel to copy.
     */
    public void setValues(GridBagBaseModel aBaseModel) {
        this.setPadding(aBaseModel.getPadding());
        this.setIpadx(aBaseModel.getIpadx());
        this.setIpady(aBaseModel.getIpady());
        this.setAnchor(aBaseModel.getAnchor());
        this.setFill(aBaseModel.getFill());
        this.setWeightx(aBaseModel.getWeightx());
        this.setWeighty(aBaseModel.getWeighty());
        this.setGridHeight(aBaseModel.getGridHeight());
        this.setGridWidth(aBaseModel.getGridWidth());
    }

    /**
     * Returns the name of the current instance of the GridBagBaseModel.
     * 
     * @return	The name of the current instance of GridBagBaseModel.
     */
    public String getName() {
        return this.m_name;
    }

    /**
     * Sets the name of the current instance of the GridBagBaseModel.
     * 
     * @parm	name	The name to use.
     */
    public void setName(String aName) {
        this.m_name = aName;
    }

    /**
     * Sets the GridWidth property that corresponds to the gridwidth attribute of the GridBagConstraints class.
     * 
     * @param	gridwidth	An int representing how many horizontal spaces a component 
     * should occupy in a GridBagLayout.
     */
    public void setGridWidth(int aGridwidth) {
        this.m_gridwidth = aGridwidth;
    }

    /**
     * Returns the GridWidth property that corresponds to the gridwidth attribute of the GridBagConstraints class.
     * 
     * @return	The number of horizontal spaces a component should occupy in a GridBagLayout.
     */
    public int getGridWidth() {
        return this.m_gridwidth;
    }

    /**
     * Sets the GridHeight property that corresponds to the gridheight attribute of the GridBagConstraints class.
     * 
     * @param	gridheight	An int representing how many vertical spaces a component 
     * should occupy in a GridBagLayout.
     */
    public void setGridHeight(int aGridheight) {
        this.m_gridheight = aGridheight;
    }

    /**
     * Returns the GridHeight property that corresponds to the gridheight attribute of the GridBagConstraints class.
     * 
     * @return	The number of vertical spaces a component should occupy in a GridBagLayout.
     */
    public int getGridHeight() {
        return this.m_gridheight;
    }

    /**
     * Sets the Weighty property that corresponds to the weighty attribute of the GridBagConstraints class.
     * 
     * @param	weighty	An int representing how much the GridBagLayout should favor
     * the current cell when rendering the vertical space it occupies.
     */
    public void setWeighty(double aWeighty) {
        this.m_weighty = aWeighty;
    }

    /**
     * Returns the Weighty property that corresponds to the weighty attribute of the GridBagConstraints class.
     * 
     * @return	An int representing how much the GridBagLayout should favor
     * the current cell when rendering the vertical space it occupies.
     */
    public double getWeighty() {
        return this.m_weighty;
    }

    /**
     * Returns the Weightx property that corresponds to the weightx attribute of the GridBagConstraints class.
     * 
     * @return	An int representing how much the GridBagLayout should favor
     * the current cell when rendering the horizontal space it occupies.
     */
    public double getWeightx() {
        return this.m_weightx;
    }

    /**
     * Sets the Weightx property that corresponds to the weightx attribute of the GridBagConstraints class.
     * 
     * @param	weightx	An int representing how much the GridBagLayout should favor
     * the current cell when rendering the horizontal space it occupies.
     */
    public void setWeightx(double aWeightx) {
        this.m_weightx = aWeightx;
    }

    /**
     * Retrieves the anchor property that corresponds to the anchor attribute of the GridBagLayout class.
     * 
     * @return An int corresponding to one of the following anchor constants exposed
     * by the GridBagConstraints class
     * 
     * @see java.awt.GridBagConstraints
     * @see java.awt.GridBagConstraints#NORTH
     * @see java.awt.GridBagConstraints#NORTHEAST
     * @see java.awt.GridBagConstraints#NORTHWEST
     * @see java.awt.GridBagConstraints#EAST
     * @see java.awt.GridBagConstraints#WEST
     * @see java.awt.GridBagConstraints#SOUTH
     * @see java.awt.GridBagConstraints#SOUTHEAST
     * @see java.awt.GridBagConstraints#SOUTHWEST
     */
    public int getAnchor() {
        return this.m_anchor;
    }

    /**
     * Sets the anchor property that corresponds to the anchor attribute of the GridBagLayout class.
     * 
     * @param	anchor	An int corresponding to one of the following anchor constants exposed
     * by the GridBagConstraints class.  This setting indicates where a component appears 
     * within its cell in a GridBagLayout.
     * 
     * @see java.awt.GridBagConstraints
     * @see java.awt.GridBagConstraints#NORTH
     * @see java.awt.GridBagConstraints#NORTHEAST
     * @see java.awt.GridBagConstraints#NORTHWEST
     * @see java.awt.GridBagConstraints#EAST
     * @see java.awt.GridBagConstraints#WEST
     * @see java.awt.GridBagConstraints#SOUTH
     * @see java.awt.GridBagConstraints#SOUTHEAST
     * @see java.awt.GridBagConstraints#SOUTHWEST
     */
    public void setAnchor(int anAnchor) {
        this.m_anchor = anAnchor;
    }

    /**
     * Retrieves the value of the Fill property.
     * 
     * @return	The value of the fill property.
     */
    public int getFill() {
        return this.m_fill;
    }

    /**
     * Set the value of the fill property.
     * 
     * @param	fill	An int representing the fill property.
     */
    public void setFill(int aFill) {
        this.m_fill = aFill;
    }

    /**
     * Retrieves the value of the Ipady property.
     * 
     * @return	An int representing the value of the Ipady property.
     */
    public int getIpady() {
        return this.m_ipady;
    }

    /**
     * Sets the value of Ipady property.
     * 
     * @param	ipady	An int representing the new value of the Ipady property.
     */
    public void setIpady(int anIpady) {
        this.m_ipady = anIpady;
    }

    /**
     * Retrieves the value of the Ipadx property.
     * 
     * @return	The value of the Ipadx property.
     */
    public int getIpadx() {
        return this.m_ipadx;
    }

    /**
     * Sets the value of the Ipadx property.
     * 
     * @param	padx	The new value of the Ipadx property.
     */
    public void setIpadx(int aPadx) {
        this.m_ipadx = aPadx;
    }

    /**
     * Retrieves the value of the Padding property.  
     * 
     * @return	The valud of the Padding property.
     */
    public int getPadding() {
        return this.m_padding;
    }

    /**
     * Sets the value of the Padding property.
     * 
     * @param	padding	The new value of the Padding property.
     */
    public void setPadding(int aPadding) {
        this.m_padding = aPadding;
    }

    /**
     * @see org.swingml.SwingMLModel
     */
    public void validate() {
    }
}