/* SwingML
 * Copyright (C) 2002 Robert Morris.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors:
 *     Robert Morris <robertj@morris.net>
 *     Ezequiel Cuellar <ecuellar@crosslogic.com> 
 *
 */

package org.swingml.model;

import java.util.List;
import java.util.Vector;

import org.swingml.Constants;
import org.swingml.SwingMLModel;


/**
 * This class holds the information used by the
 * JSplitPaneComponent class.  This class
 * can be manipulated by an implementation of InvokableEvent
 * or by an implemented script.
 * 
 * @author <a href="mailto:robertj@morris.net">Robert J. Morris</a>
 * @author <a href="mailto:ecuellar@crosslogic.com">Ezequiel Cuellar</a>
 * @author Marvin A. Dean
 *
 * @see org.swingml.component.JSplitPaneComponent
 * @see org.swingml.event.InvokableEvent
 */
public class JSplitPaneModel extends SwingMLModel {
    
    private int m_size = -1;
    private Object m_location = null;
    private List m_children = null;
    private String m_type = null;

    /**
     * Returns the type of divider orientation for the JSplitPane.
     * Will return one of the following: "Vertical" or "Horizontal".
     */
    public String getType() {
        return this.m_type;
    }

    /**
     * Set the type of divider orientation for the JSplitPane.
     * 
     * @param	type	Must be one of the following "Vertical" or "Horizontal".
     * 					In the even any other value is supplied, it is assume to be
     * 					Horizontal
     */
    public void setType(String aType) {
        if (!(aType.equalsIgnoreCase(Constants.SPLIT_HORIZONTAL) || aType.equalsIgnoreCase(Constants.SPLIT_VERTICAL))) {
            aType = Constants.SPLIT_HORIZONTAL;
        }
        this.m_type = aType;
    }

    /**
     * Returns the size of the JSplitPane's divider bar.  
     */
    public int getSize() {
        return this.m_size;
    }

    /**
     * Sets the size of the JSplitPane's divider bar.
     * 
     * @param	size	The size of the split pane's divider bar.
     */
    public void setSize(int aSize) {
        this.m_size = aSize;
    }

    /**
     * Returns the location of the JSplitPane's divider bar.  If the return
     * is an Integer instance, it represents an absolute location.  If
     * the return is a Double instance, it represents a percentage.
     */
    public Object getLocation() {
        return this.m_location;
    }

    /**
     * Sets the location of the JSplitPane's divider bar.
     * 
     * @param	location	A reference to an object.  Current, only 
     * 						Integer and Double are accepted.  Any other
     * 						type passed in, i.e. a String, causes a default
     * 						assignment of new Double(25).
     */
    public void setLocation(Object aLocation) {
        if (aLocation instanceof Integer || aLocation instanceof Double) {
            this.m_location = aLocation;
        } else {
            this.m_location = new Double(25);
        }
    }

    /**
     * @see swingml.model.Renderable#validate(ContainerModel)
     */
    public void validate() {
        if (super.getParent().getLayout() != null && super.getParent().getLayout().equalsIgnoreCase(Constants.BORDERLAYOUT)) {
            if (super.getOrientation() == null) {
                System.out.println("Syntax error: The parameter ORIENTATION in the element " + super.getName() + " is required since its parent's LAYOUT is BorderLayout. Add the parameter ORIENTATION to the element " + super.getName() + " or change its parent's LAYOUT to other than BorderLayout.");
            }
        }

        if (super.getParent().getLayout() != null && !super.getParent().getLayout().equalsIgnoreCase(Constants.BORDERLAYOUT)) {
            if (this.getOrientation() != null) {
                System.out.println("Syntax error: The parameter ORIENTATION in the element " + super.getName() + " should be used only when its parent's LAYOUT is BorderLayout. Change its parent's LAYOUT to BorderLayout or delete the parameter ORIENTATION from the element " + super.getName() + ".");
            }
        }

        if (this.getChildren().size() != 2) {
            System.out.println("Syntax error: The Split Pane [" + super.getName() + "] must contain exactly 2 components");
        }
    }

    /**
     * @see swingml.model.ContainerModel#addChild(Object)
     */
    public void addChild(Object anObject) {
        if (this.getChildren().size() < 2) {
            this.getChildren().add(anObject);
        } else {
            System.out.println("Syntax Error: The Split Pane [" + super.getName() + "] can only contain two child panels.");
        }
    }

    /**
     * @see swingml.model.ContainerModel#getChildren()
     */
    public List getChildren() {
        if (this.m_children == null) {
            this.m_children = new Vector(2);
        }
        return this.m_children;
    }
}